"""Validation submodules package for code validation.

This package provides validators for code quality and correctness.
"""

from .code_validator import CodeValidator, Violation
from .ast_validator import ASTValidator
from .python_validator import is_valid_python
from .text_analyzer import extract_code_blocks, looks_like_prompt
from .code_fixer import attempt_syntax_fix
from .quality_checker import check_code_quality
from .escape_handler import format_code_with_escape_sequences, detect_escape_sequences

__all__ = [
    'CodeValidator',
    'ASTValidator',
    'Violation',
    'is_valid_python',
    'extract_code_blocks',
    'looks_like_prompt',
    'attempt_syntax_fix',
    'check_code_quality',
    'format_code_with_escape_sequences',
    'detect_escape_sequences'
]
